/*---------------------------------------------------------------
This file is part of the Buildable Evolution Software Project. 

http://www.demo.cs.brandeis.edu/pr/buildable
(C) Pablo Funes and Jordan B. Pollack 1997-2008
pablo@cs.brandeis.edu

The methods and configurations incorporated in the Builable Evolution
Software Project are covered in part by United States patent
6,434,492 "Computer apparatus and method for analyzing structural
stability" of August 13, 2002. 

The Buildable Evolution Software Project is free software: you 
can redistribute it and/or modify it under the terms of the 
GNU General Public License as published by the Free Software 
Foundation, either version 3 of the License, or (at your option) 
any later version. 

The Buildable Evolution Software Project is distributed in the
hope that it will be useful, but WITHOUT ANY WARRANTY; without
even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more
details. 

You should have received a copy of the GNU General Public License 
along with the software. If not, see <http://www.gnu.org/licenses/>. 
---------------------------------------------------------------*/
package evocad;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Choice;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.net.URL;

import javax.swing.JApplet;

public class LegoApplet extends JApplet implements ActionListener, ItemListener{ 
	private static final long serialVersionUID = 1L;
	public boolean remoteMode=true; 
	//public static final String BaseURL="http://www.demo.cs.brandeis.edu/lego/src/v2";
	public static  /* final */ URL BaseURL = null;
	public static String extension = ".cgi";
	/* dotCgi is changed to .exe for mswindows standalone version */
	LegoCanvas canvas;
	GridBagLayout gridbag;
	GridBagConstraints c;
	Choice  weightChoice; 
	Choice SizeOfGrid;
	Choice restrictionSize;
	Button stableButton;
	Button sendButton;
	Button toggleGrid;
	Button helpButton;
	public static final boolean loadsave=true;
	private String execPrefix = null;
	int rightBoxWidth=50;
	int intHeight=LegoCanvas.intHolderHeight;
	TextArea textArea;
	static int statusLineLen=0;
	public String[]  weights = { "25","50","100","250" };
	//String t0Weight=weightChoice1;
	String t0Weight=weights[0];
	String small = "Small Structures";
	String medium = "Medium Size Structures";
	String large = "Large Structures";

	String smallRestriction="Small Restriction";
	String mediumRestriction="Long Restriction";
	String largeRestriction="Tall Restriction";


	String t0restriction = "smallRestriction";

	String t0Size=small;
	int initWeight = 25;
	NewTrash theTrash;

	void mynotify() {
		System.out.println("Start LegoApplet Remote="+remoteMode+" BaseURL="+BaseURL);
	}

	public LegoApplet(boolean remote,String url) {
		//xinit();
		remoteMode=remote;
		if (remoteMode && (url != null)) { 
			try {
				BaseURL=new URL(url);
			} catch (Exception e) {
				throw new Error("Malformed URL "+url);
			}
		}
	}

	public LegoApplet() {
		this(true,"http://www.demo.cs.brandeis.edu/lego/src/java/");
	}


	public void statusLine(String s) {
		textArea.replaceRange(s,0,statusLineLen);
		statusLineLen = s.length();
	}

	public void init() { 
		//super.init();
		setSize(1000,500);
		mynotify();

		gridbag = new GridBagLayout();
		c = new GridBagConstraints();
		Panel centerPan = new Panel();
		centerPan.setLayout(gridbag);
		statusLineLen =0;
		canvas = new LegoCanvas(this); 
		/*For Canvas*/
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 1;
		c.gridheight = 500;   
		c.anchor = GridBagConstraints.NORTHWEST;
		c.weightx =1;
		c.weighty = 1;
		// c.fill = GridBagConstraints.BOTH;
		gridbag.setConstraints(canvas, c);

		//Panels
		Panel thePan = new Panel(); 
		Panel nextTo = new Panel(new GridLayout(1,2));
		Panel nextTo2 = new Panel(new GridLayout(1,2));

		//Choices
		SizeOfGrid = new Choice(); 
		SizeOfGrid.addItem(small);	     
		SizeOfGrid.addItem(medium);
		SizeOfGrid.addItem(large);
		restrictionSize= new Choice();
		restrictionSize.add(smallRestriction);
		restrictionSize.add(mediumRestriction);
		restrictionSize.add(largeRestriction);
		weightChoice = new Choice();
		for (int i=0;i<weights.length;i++){ 
			weightChoice.addItem(weights[i]);
		}

		//Buttons
		Button saveButton = new Button("Save");
		Button loadButton = new Button("Load");	
		Button clearButton = new Button("Clear All");
		Button Clear_Bricks = new Button("Clear Bricks");
		stableButton = new Button("Check Stability");
		sendButton = new Button("Evolve Structure");
		textArea = new TextArea(2, 1);
		statusLine("Start Building");
		toggleGrid = new Button("Toggle Grid");
		helpButton = new Button("Help");
		//Adding Buttons and Choices to Panels
		thePan.setLayout(new GridLayout(9,1));
		theTrash = new NewTrash(canvas);
		thePan.add(weightChoice);
		thePan.add(SizeOfGrid);
		thePan.add(restrictionSize);
		thePan.add(stableButton);
		thePan.add(sendButton);
		thePan.add(toggleGrid);
		thePan.add(helpButton);
		nextTo2.add(saveButton);
		nextTo2.add(loadButton);
		nextTo.add(clearButton);
		nextTo.add(Clear_Bricks);
		thePan.add(nextTo);
		thePan.add(nextTo2);

		thePan.setSize(thePan.getMinimumSize());
		Panel newPan = new Panel();
		newPan.setLayout(new BorderLayout());
		Panel eastPan = new Panel();
		eastPan.setLayout(new GridLayout(2,1));
		eastPan.add(thePan, "North");
		eastPan.add(theTrash, "Center");
		centerPan.add(canvas);
		centerPan.add(eastPan);
		//centerPan.add(textArea);
		newPan.add(centerPan, "Center");
		newPan.add(textArea, "South");
		add(newPan);

		//Action and item Listeners
		saveButton.addActionListener(this);	
		loadButton.addActionListener(this);
		clearButton.addActionListener(this);
		Clear_Bricks.addActionListener(this);
		stableButton.addActionListener(this);
		helpButton.addActionListener(this);
		sendButton.addActionListener(this);
		weightChoice.addItemListener(this);
		SizeOfGrid.addItemListener(this);
		toggleGrid.addActionListener(this);
		restrictionSize.addItemListener(this);
		//canvas.enable(); 
		repaint();
	}


//	public void update( Graphics g ) { 
//		paint(g);
//	}  

	public void actionPerformed(ActionEvent e) {
		if ("Evolve Structure" == e.getActionCommand())
			canvas.evolve();   
		else if ("Check Stability" == e.getActionCommand())
			canvas.lTest();  
		else if ("Clear Bricks" == e.getActionCommand())
			canvas.Empty_Bricks(true);	  
		else if ("Load" == e.getActionCommand())
			canvas.load_saved();
		else if ("Save" == e.getActionCommand())
			canvas.save_all();
		else if ("Clear All" == e.getActionCommand())
			canvas.clearAllBricks();
		else if ("Help" == e.getActionCommand()){
			try{ 
				URL url = new URL ("http://www.demo.cs.brandeis.edu/pr/buildable/evocad/Lous_Instructions.html");
				getAppletContext().showDocument(url, "Help Window"); 
			} catch (Exception ex)	 {System.out.println("Couldn't find help file exception="+ex);}
		}else if ("Toggle Grid" == e.getActionCommand()){
			if (canvas.showGrid == true)
				canvas.showGrid = false;
			else canvas.showGrid=true;
			canvas.repaint();
		}
	}

	public void itemStateChanged(ItemEvent e) {
		double height=0;
		String t1Weight= weightChoice.getSelectedItem();
		String t1Size = SizeOfGrid.getSelectedItem();
		String t1restriction = restrictionSize.getSelectedItem();
		int weight=0;

		if (t0Weight != t1Weight){
			for(int i=0;i<weights.length;i++) {
				if (weightChoice.getSelectedItem() == weights[i]) {
					height=(i+1)*canvas.Lego_Height_Constant;
					weight =(int) Integer.parseInt(weightChoice.getSelectedItem()); 
					break;
				}
			}
			intHeight = (int) Math.floor(height);

			for (int i=0; i<canvas.intObject_Counter; i++){
				if ((canvas.Movable_Objects[i].whichObject == LegoCanvas.WEIGHT) && canvas.Movable_Objects[i].blnNewUser_Object){	
					canvas.Movable_Objects[i].legoHeight = intHeight;  
					canvas.Movable_Objects[i].weight = weight;  
					canvas.updateWeight = i;
					break;
				}
			}
			canvas.repaint();
			t0Weight = weightChoice.getSelectedItem();
		}else if (t0Size != t1Size){
			if (SizeOfGrid.getSelectedItem() == small){  
				canvas.changeGridSize(1);
			}else if (SizeOfGrid.getSelectedItem() == medium){  
				canvas.changeGridSize(2);
			}else if (SizeOfGrid.getSelectedItem() == large){  
				canvas.changeGridSize(3);
			}
			t0Size = SizeOfGrid.getSelectedItem();	      
		}else if (t0restriction != t1restriction){
			if (restrictionSize.getSelectedItem() == smallRestriction){  
				canvas.changeRestrictionSize(1);
			}else if (restrictionSize.getSelectedItem() == mediumRestriction){  
				canvas.changeRestrictionSize(2);
			}else if (restrictionSize.getSelectedItem() == largeRestriction){  
				canvas.changeRestrictionSize(3);
			}
			t0restriction = SizeOfGrid.getSelectedItem();	     
		}      
	}

	public void setExtension(String ext) {
		extension = ext;
	}
	
	public boolean isRemoteMode() {
		return remoteMode;
	}

	public void setRemoteMode(boolean remoteMode) {
		this.remoteMode = remoteMode;
	}

	public void paint(Graphics g) { 
		if (canvas == null) {
			init();
			canvas.paint(g);
		}

	}

	public String getExecPrefix() {
		return execPrefix;
	}

	public void setExecPrefix(String execPrefix) {
		this.execPrefix = execPrefix;
	}

}

