/*---------------------------------------------------------------
This file is part of the Buildable Evolution Software Project. 

http://www.demo.cs.brandeis.edu/pr/buildable
(C) Pablo Funes and Jordan B. Pollack 1997-2008
pablo@cs.brandeis.edu

The methods and configurations incorporated in the Builable Evolution
Software Project are covered in part by United States patent
6,434,492 "Computer apparatus and method for analyzing structural
stability" of August 13, 2002. 

The Buildable Evolution Software Project is free software: you 
can redistribute it and/or modify it under the terms of the 
GNU General Public License as published by the Free Software 
Foundation, either version 3 of the License, or (at your option) 
any later version. 

The Buildable Evolution Software Project is distributed in the
hope that it will be useful, but WITHOUT ANY WARRANTY; without
even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more
details. 

You should have received a copy of the GNU General Public License 
along with the software. If not, see <http://www.gnu.org/licenses/>. 
---------------------------------------------------------------*/
package evocad;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Point;
import java.awt.Toolkit;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.Random;
import java.util.StringTokenizer;

/**
 * This File is the canvas for the applet.
 * The file also controls the mouse commands and the communication to the CGI's.
 */

public class LegoCanvas extends Canvas {   
	/*Next three lines are initializations for media tracker, making sure all images load up */	  
	LegoApplet applet;   
	MediaTracker tracker;   
	boolean blnShow = false;	

	int maxObjects = 1500; //Max number of bricks allowed on grid	
	private static Grid grid;	
	int startObjectCount=10;
	int intObject_Counter = startObjectCount;//This number represents the number of objects you start out with	 
	int groundIndex = intObject_Counter -1;  //***was intObjectCounter - 1;   
	int intNum_Non_Bricks= 3; //number of non bricks, restrictions, circles, arrows.	
	int menu_Items = 10;	  //the number of objects painted for initialization   
	int numOfImages = 17;
	int updateWeight = 0;   
	//Applet specs	
	public static int intAppletHeight = 480;   
	public static int intAppletWidth =	900;   

	//Grid Coords   
	public static int intGridWidth = 40;//was 45   
	public static int intGridHeight = 20 ;//was 23				    
	public static int intHolderWidth = 18;   //was 15
	public static int intHolderHeight = 18;//was 15
	public static int intGridX1 = 8;//was 0
	public static int intGridX2 = intGridWidth * intHolderWidth-8; //675  was this + gridX1
	public static int intGridY1 = 47;	
	//the height of the grid is 345   
	public static int intGridY2 = intGridY1 + (intGridHeight * intHolderHeight);//385  
	public static int xMin = 0;
	public static int yMin = 0;

	int canvasHeight=intGridY2+2;//was gridY2+10
	int canvasWidth=intGridX2;//was gridX2 + 10

	public static final int GROUND = 0;	  
	public static final int BRICK = 1;	 
	public static final int TARGET_POINT = 2;	
	public static final int RESTRICTION = 3;   
	public static final int WEIGHT = 4;	  
	public static final int FIXED = 5;	
	public static final int GRID_SIZE = 100;

	int textX = 100; 
	int textY = 200; 
	int textWidth = 300; 
	int textHeight = 60; 

	//These next two constants make it easy to change the witdth and height of the bricks   
	int Lego_Height_Constant = intHolderHeight+2;   
	int Lego_Width_Multiple = intHolderWidth;	

	//For Save Data:   
	int NEW_STRUCTURE = 0;   
	int CHECK_FOR_FALLING_BRICKS = 1; 

	//Other Booleans   
	boolean blnMaking_New_Brick = false;   
	boolean loading;	 

	//images of everything except the building bricks and all the other bricks	 
	public static Image happyFace;   
	public static Image sadFace;   
	public static Image weight;	  
	public static Image waves;	  

	//These are creations of objects:	
	//Movable_Objects is the array which holds objects such as bricks and starting brick.   
	public UserObject Movable_Objects [];   
	public ImageHolder Image_List [] = new ImageHolder[numOfImages];   
	UserObject Temp_Array []; 
	Trash trash = new Trash();	 
	//New random for weight placement (not currently used)	
	Random r = new Random();   

	//Inits for Resizing Grid
	boolean placedFirstBrick=false;
	//Boolean to indicate wether grid lines should be displayed
	boolean showGrid = false;

	Graphics big;
	Image bi;
	boolean firstTime = true;

	/* Class ioObject, myTokenizer, and structParser are used to make IO easier*/
	class ioObject {
		public int kind,x,y,size;
		public ioObject(final int k,final int x,final int y, final int s) {
			this.kind=k;
			this.x=x;
			this.y=y;
			this.size=s;
		}
	}


	class myTokenizer {
		BufferedReader in;
		public String s;
		public StringTokenizer st;
		public int tokenshere=0;

		public myTokenizer(final BufferedReader in) {
			this.in = in;
		}

		public void nextline() throws IOException {
			s = "";
			tokenshere=0;
			while (tokenshere == 0) {
				s = in.readLine();
				if (s == null) throw new IOException("End-of-file");
				if (s.length() > 0 && s.charAt(0) != '#') {
					st = new StringTokenizer(s);
					tokenshere = st.countTokens();
				}
			}
			return;
		}

		public int nextInt () throws NumberFormatException, IOException	 {
			if (tokenshere <=0) 
				nextline();

			tokenshere--;
			return Integer.parseInt(st.nextToken());
		}

	}
	class StructParser {
		public boolean valid=false;
		public String errorMsg = "unread object";
		public int objectCount=-1;
		public ioObject[] objectList;


		public StructParser(final BufferedReader in) {
			int i;
			int ok,ox,oy,os;
			final myTokenizer tok=new myTokenizer(in);
			try { 
				errorMsg="syntax error: no tokens";
				objectCount=tok.nextInt();
				if (objectCount<=0) {
					errorMsg = tok.s;
				}
				else {
					errorMsg="Incomplete Object List";
					objectList=new ioObject[objectCount];
					for (i=0;i<objectCount;i++) {
						ok=tok.nextInt();
						ox=tok.nextInt();
						oy=tok.nextInt();
						os=tok.nextInt();
						objectList[i]=new ioObject(ok,ox,oy,os);
					}
					valid=true;
					errorMsg="valid object";
				}
			}

			catch (final Exception e)	 {
				objectCount=-1;
				valid=false;
				objectList=null;
				errorMsg=e+errorMsg;
			}

			if (valid) applet.statusLine("Structure Loaded");
			else applet.statusLine("Structure not loaded: "+errorMsg);
		}
	}


	Image findImage(final String name) throws MalformedURLException {
		Image im;
		if (applet.isRemoteMode()) {
			im = getToolkit().getImage(new URL(LegoApplet.BaseURL+name));
		}
		else {
			final String imgPath = "/img/"+name;
			final URL imgResource = getClass().getResource(imgPath);
			if (imgResource == null) throw new Error("Resource "+imgPath+" not found.");
			im=Toolkit.getDefaultToolkit().getImage(imgResource);
		}
		return im;
	}


	public LegoCanvas( final LegoApplet applet ) {   
		this.applet = applet;  
		/*IMAGE INITS*/
		for (int i = 0; i < numOfImages; i++){ 
			Image_List[i] = new ImageHolder(); 
		} 

		try { 
			Image_List[0].theImage = findImage( "1bump.gif");	
			Image_List[1].theImage = findImage( "circle.gif");	 
			Image_List[2].theImage =  findImage( "BuildingBlock4.gif");  
			Image_List[3].theImage  =findImage( "BuildingBlock4.gif");	 
			Image_List[4].theImage  = findImage( "1bump.gif");	 
			Image_List[5].theImage  = findImage( "2bump.gif");	 
			Image_List[6].theImage  =findImage( "4bump.gif");	
			Image_List[7].theImage  = findImage("6bump.gif");	
			Image_List[8].theImage = findImage("8bump.gif");  
			Image_List[9].theImage  = findImage("bump10.gif");	 
			Image_List[10].theImage = findImage("1bumpBreak.gif");  
			Image_List[11].theImage =findImage("2bumpBreak.gif");  
			Image_List[12].theImage = findImage( "4bumpBreak.gif");  
			Image_List[13].theImage = findImage( "6bumpbreak.gif");  
			Image_List[14].theImage =findImage( "8bumpbreak.gif");  
			Image_List[15].theImage = findImage("bump10break.gif");  
			Image_List[16].theImage = findImage( "trash.gif");	 
		}	
		catch ( final MalformedURLException murlx ) { 
			System.out.println("exception caught "+ murlx); 
		}	    
		//MEDIA TRACKER	  
		//These next few lines gurantee that all the images load up before they are painted   
		//by using a media tracker   
		tracker = new MediaTracker(this);  
		//This for loop is for the bricks   
		for (int i =0; i< numOfImages; i++){   
			tracker.addImage(Image_List[i].theImage, i);   
			if (Image_List[i].theImage == null) System.out.println("ImageNull="+i);
		}   
		try {	
			for (int k =0; k < numOfImages; k++){	   

				tracker.waitForID( k );	  
			}	
		} catch( final InterruptedException e) { System.out.println("EXCEPTION IN MEDIA TRACKER="+e);}   
		if ( tracker.isErrorID( 0 ) ){	
			System.out.println("Images did not load up correctly, please make sure you have all the images");	
		}else	
			blnShow = true;   

		//Now Initialize Everything Else
		myInit();
	}	

	public Dimension minimumSize() { 

		return( new Dimension( canvasWidth, canvasHeight)); 

	} 


	public Dimension preferredSize() { 

		return( minimumSize() ); 

	}

	public void myInit(){
		grid = new Grid(applet.canvas);
		Movable_Objects= new UserObject[maxObjects]; 
		Temp_Array = new UserObject[maxObjects];//Temp array is used to paint User_Objects from bottom up	    

		startObjectCount=10;
		intObject_Counter = startObjectCount;//This is the number represents the number of objects you start out with	 
		groundIndex = intObject_Counter -1;  //***was intObjectCounter - 1;   
		intNum_Non_Bricks= 3; //number of non bricks, restrictions, circles, arrows.	
		menu_Items = 10;	  //the number of objects painted for initialization   
		numOfImages = 17;
		updateWeight = 0;   
		//These next two constants make it easy to change the witdth and height of the bricks	
		Lego_Height_Constant = intHolderHeight+2;   
		Lego_Width_Multiple = intHolderWidth;	
		//For Save Data:   
		NEW_STRUCTURE = 0;   
		CHECK_FOR_FALLING_BRICKS = 1; 
		//Other Booleans		     
		blnMaking_New_Brick = false;	 
		//for non bricks   
		Movable_Objects[0] = new UserObject();	  
		Movable_Objects[1] = new UserObject();	  
		Movable_Objects[2] = new UserObject();	  

		int multiple = 0;   
		int prev_init = 9;   
		int hldr_cntr = 0;   
		//MAKES NEW BRICKS, initializes INIT and Before variables   
		for (int j = intNum_Non_Bricks; j < groundIndex; j++){	 
			Movable_Objects[j] = new UserObject();   
			Movable_Objects[j].initX = prev_init+ (multiple * 30);   
			Movable_Objects[j].initY = 10;   
			Movable_Objects[j].beforeX = Movable_Objects[j].initX;   
			Movable_Objects[j].beforeY = Movable_Objects[j].initY;   
			Movable_Objects[j].lego1X = Movable_Objects[j].beforeX;   
			Movable_Objects[j].lego1Y = Movable_Objects[j].beforeY;  
			if (hldr_cntr == 0){   
				Movable_Objects[j].holderSizeInX = (hldr_cntr + 1);   
				Movable_Objects[j].legoWidth = Lego_Width_Multiple;   
			}	
			else{   
				Movable_Objects[j].holderSizeInX = (hldr_cntr);	
				Movable_Objects[j].legoWidth = Lego_Width_Multiple * Movable_Objects[j].holderSizeInX;	  
			}	  
			Movable_Objects[j].legoHeight =  Lego_Height_Constant;   
			Movable_Objects[j].whichObject = BRICK;   
			prev_init = Movable_Objects[j].initX;   
			hldr_cntr = hldr_cntr + 2;	 
			multiple ++;      
		}   

		//FOR NON BRICK initializes   
		//This is for the building block   
		Movable_Objects[groundIndex] = new UserObject();   
		Movable_Objects[groundIndex].initX = 350;//The initial coords of the building block   
		Movable_Objects[groundIndex].initY = 29;//Movable_Objects[intNum_Non_Bricks+2].initY+Lego_Height_Constant+5;   
		Movable_Objects[groundIndex].beforeX = Movable_Objects[groundIndex].initX;   
		Movable_Objects[groundIndex].beforeY = Movable_Objects[groundIndex].initY;	 
		Movable_Objects[groundIndex].holderSizeInX = 8;	  
		Movable_Objects[groundIndex].legoWidth = (Lego_Width_Multiple * Movable_Objects[groundIndex].holderSizeInX);   
		Movable_Objects[groundIndex].legoHeight = Lego_Height_Constant;	  
		Movable_Objects[groundIndex].lego1X = Movable_Objects[groundIndex].beforeX;   
		Movable_Objects[groundIndex].lego1Y = Movable_Objects[groundIndex].beforeY;   
		Movable_Objects[groundIndex].whichObject = GROUND;   
		Movable_Objects[groundIndex].blnNewUser_Object = true;	  

		//This is the inits for the X	
		Movable_Objects[0].initX = intGridX2-20;   
		Movable_Objects[0].initY = 10;	 
		Movable_Objects[0].beforeX = Movable_Objects[0].initX;	 
		Movable_Objects[0].beforeY = Movable_Objects[0].initY;	 
		Movable_Objects[0].holderSizeInX = 1;	
		Movable_Objects[0].legoWidth = (Lego_Width_Multiple * Movable_Objects[0].holderSizeInX);   
		Movable_Objects[0].legoHeight = Lego_Height_Constant;	
		Movable_Objects[0].lego1X = Movable_Objects[0].beforeX;	  
		Movable_Objects[0].lego1Y = Movable_Objects[0].beforeY;	  
		Movable_Objects[0].whichObject = RESTRICTION;	
		Movable_Objects[0].blnNewUser_Object = true;   
		//This is the inits for the o	
		Movable_Objects[1].initX = intGridX2-35;   
		Movable_Objects[1].initY = 10;	 
		Movable_Objects[1].beforeX = Movable_Objects[1].initX;	 
		Movable_Objects[1].beforeY = Movable_Objects[1].initY;	 
		Movable_Objects[1].holderSizeInX = 1;	
		Movable_Objects[1].legoWidth = (Lego_Width_Multiple * Movable_Objects[1].holderSizeInX);   
		Movable_Objects[1].legoHeight = Lego_Height_Constant;	
		Movable_Objects[1].lego1X = Movable_Objects[1].beforeX;	  
		Movable_Objects[1].lego1Y = Movable_Objects[1].beforeY;	  
		Movable_Objects[1].blnNewUser_Object = true;   
		Movable_Objects[1].whichObject = TARGET_POINT;	 
		//For The weight
		Movable_Objects[2].initX = intGridX2-50;   
		Movable_Objects[2].initY = 10;	 
		Movable_Objects[2].beforeX = Movable_Objects[2].initX;	 
		Movable_Objects[2].beforeY = Movable_Objects[2].initY;	 
		Movable_Objects[2].holderSizeInX = 1;	
		Movable_Objects[2].legoWidth = (Lego_Width_Multiple * Movable_Objects[1].holderSizeInX);   
		Movable_Objects[2].legoHeight = Lego_Height_Constant;	
		Movable_Objects[2].lego1X = Movable_Objects[2].beforeX;	  
		Movable_Objects[2].lego1Y = Movable_Objects[2].beforeY;	  
		Movable_Objects[2].blnNewUser_Object = true;   
		Movable_Objects[2].whichObject = WEIGHT; 
		Movable_Objects[2].weight =(int) Integer.parseInt(applet.weights[0]);


		trash.trashImage = Image_List[16].theImage ;	   
		//Images of restriction and targe point
		Movable_Objects[0].Lego_Image = get_Image(1, RESTRICTION, false);   
		Movable_Objects[1].Lego_Image = get_Image(1, TARGET_POINT, false);
		//Movable_Objects[2].Lego_Image = get_Image(1, WEIGHT, false);	 
		//Movable_Objects[2].Lego_Image = get_Image(4, FIXED, false);	
		Movable_Objects[groundIndex].Lego_Image = get_Image(8, GROUND, false);	
		Movable_Objects[intNum_Non_Bricks].Lego_Image = get_Image(1, BRICK, false);   
		Movable_Objects[intNum_Non_Bricks +1].Lego_Image = get_Image(2, BRICK, false);	 
		Movable_Objects[intNum_Non_Bricks+2].Lego_Image = get_Image(4,BRICK, false);   
		Movable_Objects[intNum_Non_Bricks+3].Lego_Image = get_Image(6,BRICK, false);   
		Movable_Objects[intNum_Non_Bricks+4].Lego_Image = get_Image(8, BRICK, false);	
		Movable_Objects[intNum_Non_Bricks+5].Lego_Image = get_Image(10, BRICK, false);	 

		//break images, they are black, there is not break image for the 0'th brick because    
		//it is the building brick   
		Movable_Objects[intNum_Non_Bricks].Break_Image = get_Image(1, BRICK, true);   
		Movable_Objects[intNum_Non_Bricks+1].Break_Image = get_Image(2, BRICK, true);	
		Movable_Objects[intNum_Non_Bricks+2].Break_Image = get_Image(4, BRICK, true);	
		Movable_Objects[intNum_Non_Bricks+3].Break_Image = get_Image(6, BRICK, true);	
		Movable_Objects[intNum_Non_Bricks+4].Break_Image =get_Image(8, BRICK, true);   
		Movable_Objects[intNum_Non_Bricks+5].Break_Image = get_Image(10,BRICK , true);	 

	}


	//Any time a user is clicking on a brick, then its location must be updated on the screen.	 
	//Every brick is checked to see if it is being clicked on, if it is, then its location is	 
	//updated (updateLocation) and then repainted (repaint());	 
	public boolean mouseDrag(final Event evt,final int x, final int y){	
		blnMaking_New_Brick = false;   
		for (int i =0; i< intObject_Counter; i++){   
			if (Movable_Objects[i].blnClickinOn){		   
				updateLocation(x,y, Movable_Objects[i]);   
				repaint();   
			}	
		}   
		return true;   
	}	

	//This function takes a brick and coordinates for a rectangular object, a grid in this case.   
	//It returns true if any part of the brick is inside the rectangle.	  
	public boolean OverGrid(final UserObject User_ObjectX, final int xMin, final int yMin, final int xMax, final int yMax ){	  
		final boolean X1Within = ((xMin <= User_ObjectX.lego1X) && (User_ObjectX.lego1X <= xMax)) ;	
		final boolean X2Within = ((xMin <= (User_ObjectX.lego1X + User_ObjectX.legoWidth)) && ((User_ObjectX.lego1X + User_ObjectX.legoWidth) <= xMax));   
		final boolean Y1Within = ((yMin <= User_ObjectX.lego1Y) && (User_ObjectX.lego1Y <= yMax));   
		final boolean Y2Within = ((yMin <= (User_ObjectX.lego1Y + User_ObjectX.legoHeight)) && ((User_ObjectX.lego1Y + User_ObjectX.legoHeight) <= yMax));   

		if ( ((X1Within)  || (X2Within))  && ((Y1Within) || (Y2Within)))   
			return true;   
		else   
			return false;   
	}	



	//In MouseDown every object that can be clicked on, must be checked.     
	//There are two objects I need to check	  
	public boolean mouseDown(final Event evt,final int x, final int y){	


		blnMaking_New_Brick = false;
		boolean clickingOnOne = true;
		//This next for-loop scans all the bricks and finds the pieces that have been clicked on	   
		for (int j = 0; j<  getObjectListLength(); j++){   
			final int xMin = Movable_Objects[j].lego1X;	
			final int yMin = Movable_Objects[j].lego1Y;	
			final int xMax = xMin + Movable_Objects[j].legoWidth;	  
			final int yMax = yMin + Movable_Objects[j].legoHeight;   
			if (Contain(x, y, xMin, xMax, yMin, yMax) && clickingOnOne){	  
				Movable_Objects[j].blnClickinOn = true;	  
				Movable_Objects[j].xclick = x - Movable_Objects[j].lego1X;
				Movable_Objects[j].yclick = y - Movable_Objects[j].lego1Y;  
				//If the brick is not a new brick, then we must erase the old spots it took up
				if (Movable_Objects[j].blnNewUser_Object == false){   
					emptyGridHolders(Movable_Objects[j]);			   
				}   
				clickingOnOne = false;
			}	
			else {   
				Movable_Objects[j].blnClickinOn = false;   
				Movable_Objects[j].xclick = Movable_Objects[j].yclick=0;   
			}	
		}   
		return true;   
	}	

	//Function takes a Brick as input.	   
	void emptyGridHolders(final UserObject object){	   
		final int whichObject = object.whichObject;	
		final int intXClose =	 (int) Math.round( (double)  (object.lego1X - intGridX1 ) / intHolderWidth );	
		final int intYClose =	 (int) Math.round(  (double) (object.lego1Y - intGridY1 ) / intHolderHeight);	
		for (int w = 0; w < object.holderSizeInX; w++){	  
			if ((w + intXClose < intGridWidth) && (intXClose+w >= 0)){	 
				if (whichObject == BRICK)   
					grid.theGrid[intXClose+w][intYClose].FullWithABrick = false;   
				else if (whichObject == RESTRICTION)	
					grid.theGrid[intXClose+w][intYClose].FullWithARestriction = false;	 
				else if (whichObject == TARGET_POINT)	
					grid.theGrid[intXClose+w][intYClose].FullWithATarget = false;   
				else if (whichObject == GROUND)	  
					grid.theGrid[intXClose+w][intYClose].FullWithGround = false;   
				else if (whichObject == WEIGHT)
					grid.theGrid[intXClose+w][intYClose].FullWithAWEIGHT = false;
			}	
		} 
		for (int w = 0; w < object.holderSizeInY; w++){	  
			if ((w + intYClose < intGridHeight) && (intYClose+w >= 0)){	 
				if (whichObject == BRICK)   
					grid.theGrid[intXClose][intYClose+w].FullWithABrick = false;   
				else if (whichObject == RESTRICTION)	
					grid.theGrid[intXClose][intYClose+w].FullWithARestriction = false;	 
				else if (whichObject == TARGET_POINT)	
					grid.theGrid[intXClose][intYClose+w].FullWithATarget = false;   
				else if (whichObject == GROUND)	  
					grid.theGrid[intXClose][intYClose+w].FullWithGround = false;   
				else if (whichObject == WEIGHT)
					grid.theGrid[intXClose][intYClose+w].FullWithAWEIGHT = false;
			}	
		}     
	}	

	/*Function takes an object as input, true is returned if no collision is detected.	 
      Each object has different rules for collisions   
      	Target Points- Target points can collide with bricks   
      	Bricks- Bricks can collide with target points, nothing else. etc...   
        Restrictions-can't collide with anything
        loads-can collide with bricks only
        Ground-same as bricks, except loads and target points 
      Purpose:Mostly used to check if one brick is on top of another, but can be extended to check if	
      anything is occupying an area of the grid	  
	 */
	public boolean NoCollisionDetected(final UserObject object){   
		int xClose = (int) Math.round( (double) (object.lego1X - intGridX1 ) / intHolderWidth ) ;   
		int yClose = (int) Math.round( (double) (object.lego1Y - intGridY1 ) / intHolderHeight );   
		//The next 4 if will make the brick fall into place if the brick is placed   
		//just slightly off of the grid.   
		if ((xClose + object.holderSizeInX) > intGridWidth){   
			xClose = intGridWidth - object.holderSizeInX;	 
		}      
		else if (xClose < 0)   
			xClose = 0;	  

		if (yClose >= intGridHeight){	
			yClose = intGridHeight -1;	 
		}   

		else if (yClose < 0) {	 
			yClose = 0;		   
		}		  
		//Weights behaves exactly like target points		    
		if ((object.whichObject == TARGET_POINT) || (object.whichObject == WEIGHT) ){	
			for (int k = 0; k < object.holderSizeInX; k++){   
				if ((xClose + k < intGridWidth) && (xClose +k >= 0)){	
					if ((yClose < intGridHeight) && (yClose >= 0))  {	
						if (grid.theGrid[xClose + k][yClose].FullWithATarget)	
							return false; 
						else if (grid.theGrid[xClose + k][yClose].FullWithAWEIGHT)   
							return false;      
						else if (grid.theGrid[xClose + k][yClose].FullWithARestriction)	  
							return false;   
						else if (grid.theGrid[xClose + k][yClose].FullWithGround)   
							return false;   
					}	
				}   
			}	
			return true;   
		}   
		else {	 
			if (object.whichObject == BRICK){	
				for (int k = 0; k < object.holderSizeInX; k++){	  
					if ((xClose + k < intGridWidth) && (xClose +k >= 0)){   
						if ((yClose < intGridHeight) && (yClose >= 0)){	  
							//if a holder is not full with a target, but it is full with something else, return false	
							if (grid.theGrid[xClose + k][yClose].FullWithARestriction)	  
								return false;	
							else if (grid.theGrid[xClose + k][yClose].FullWithABrick)	
								return false;	
							else if (grid.theGrid[xClose + k][yClose].FullWithGround)  
								return false;	     
						}   
					}	
				}   
				return true;		   
			} else if (object.whichObject == GROUND){	
				for (int k = 0; k < object.holderSizeInX; k++){	  
					if ((xClose + k < intGridWidth) && (xClose +k >= 0)){   
						if ((yClose < intGridHeight) && (yClose >= 0)){	  
							//if a holder is not full with a target, but it is full with something else, return false	
							if (grid.theGrid[xClose + k][yClose].FullWithARestriction)	  
								return false;	
							else if (grid.theGrid[xClose + k][yClose].FullWithABrick)	
								return false;	
							else if (grid.theGrid[xClose + k][yClose].FullWithGround)  
								return false;	     
							else if (grid.theGrid[xClose + k][yClose].FullWithATarget)	
								return false;	
							else if (grid.theGrid[xClose + k][yClose].FullWithAWEIGHT)	
								return false;	
						}   
					}	
				}   

				return true;		   
			}	
			else { 

				for (int k = 0; k < object.holderSizeInX; k++){	  
					if ((xClose + k < intGridWidth) && (xClose +k >= 0)){   
						if ((yClose < intGridHeight) && (yClose >= 0)){	  
							if ( (grid.theGrid[xClose + k][yClose].FullWithATarget)  ||	  
									(grid.theGrid[xClose + k][yClose].FullWithABrick)   ||	  
									(grid.theGrid[xClose + k][yClose].FullWithGround)  ||	 
									(grid.theGrid[xClose + k][yClose].FullWithARestriction) )   
								return false;	
						}   
					}	
				}
				for (int k = 0; k < object.holderSizeInY; k++){	  
					if ((yClose + k < intGridHeight) && (yClose +k >= 0)){   
						if ((xClose < intGridWidth) && (xClose >= 0)){	  
							if ( (grid.theGrid[xClose][yClose+k].FullWithATarget)  ||	  
									(grid.theGrid[xClose][yClose+k].FullWithABrick)   ||	  
									(grid.theGrid[xClose][yClose+k].FullWithGround)  ||	 
									(grid.theGrid[xClose][yClose+k].FullWithARestriction) )   
								return false;	
						}   
					}	
				}	   
				return true;   
			}	   
		} 
	}	

	/*When a mouse up event occurs, and you were clicking on a brick right before you put the mouse up, then   
      check two things:	  
      A. If a brick was dropped in the grid and not on top of another piece (a legal move)    
      THEN-fill the grid with that brick; but we also must check 2 more things.	  
      1. If it was a New Brick: A new brick with the same attributes as the brick just placed on the	
      board is made.   
      2. If it was already on the grid: No new brick is made	

      B. Else, the move was illegal.	 
      1. It was a New Brick- move it back to its initial coords.   
      2. It was already on the grid- move it back to the spot it was just in.		   
      Purpose: To move the brick to an appropriate place when it is dropped.*/	 
	public boolean mouseUp(final Event evt,int x,int y){   
		final int lastBrick = getObjectListLength();	  
		for(int l = 0; l<  lastBrick; l++){   
			if (Movable_Objects[l].blnClickinOn) {   
				//find out what the hell we are clicking on   
				//check all for overgrid,   
				//treat x like a brick	 
				//treat o like a brick ecept collisions are detected, dont fill grid   
				//do we needMovable_Objects to use these two lines below??   
				placedFirstBrick = true;
				x = x - Movable_Objects[l].xclick;   
				y = y - Movable_Objects[l].yclick;   
				//If it is an oo, then	  
				//it can collide with other bricks   
				//it can't colide with other o's or x's	 
				if (OverGrid(Movable_Objects[l], intGridX1, intGridY1, intGridX2, intGridY2) &&	  
						NoCollisionDetected(Movable_Objects[l])){	 
					Get_Closest_Holder_and_Fill_Grid(Movable_Objects[l].lego1X, Movable_Objects[l].lego1Y, Movable_Objects[l]);		 
					//If it is a new brick then we must make a new brick that	 
					//has the same exact attributes of the last brick, and put it in the top	
					//part of the applet.   
					if (Movable_Objects[l].blnNewUser_Object){	
						//First make the last object in the list equal to the lth object on the list	
						Movable_Objects[lastBrick] = Movable_Objects[l];   
						//Indicate that it is not a new object any more, rather it is off of the menu	
						Movable_Objects[lastBrick].blnNewUser_Object = false;  


						//size = Movable_Objects[lastBrick].holderSizeInX;
						int yParameter = Movable_Objects[lastBrick].holderSizeInY;
						//For weights, the holdersize in the Y direction is 1, not the actual height
						if (Movable_Objects[lastBrick].whichObject == WEIGHT)
							yParameter= (int) Integer.parseInt(applet.weightChoice.getSelectedItem());  


						//Make a new object of the lth item in the list, but make it a menu item.   
						Movable_Objects[l] = MakeANewObject(Movable_Objects[lastBrick].whichObject,   
								Movable_Objects[lastBrick].initX,	 
								Movable_Objects[lastBrick].initY,	 
								Movable_Objects[lastBrick].holderSizeInX,
								yParameter,	  
								true);//True because it is a menu item    
						intObject_Counter = intObject_Counter + 1;   
						blnMaking_New_Brick = true;   
					}	
				}   
				else {	     
					if (Movable_Objects[l].blnNewUser_Object){	 
						Movable_Objects[l].nextX = Movable_Objects[l].initX;   
						Movable_Objects[l].nextY = Movable_Objects[l].initY;   
					}	
					else{//It is not a new object, two things could have happened  
						final Point point =  applet.theTrash.getLocOnScreen();
						if (Contain(x, y, point.x, point.x+100, point.y, point.y+125)){	 
							if (l == (lastBrick -1)){	
								Movable_Objects [l] = null;   
								intObject_Counter = lastBrick - 1;
							}	
							else{ 
								Movable_Objects [l]= Movable_Objects [lastBrick -1];  
								Movable_Objects [lastBrick -1] = null; 
								intObject_Counter = lastBrick - 1;	
								repaint(); 
								break;	 
							}	
						}    
						else {	 
							Get_Closest_Holder_and_Fill_Grid(Movable_Objects[l].beforeX, Movable_Objects[l].beforeY, Movable_Objects[l]);	  
						}   
					}	
				} 
				repaint(); 
			}	
		}	  
		return true;   
	}	

	//Takes one brick as input, finds the closest holders, and fills those spots.   
	//Every holder gets filled with a "filled with some object"	  
	//But the holders can also be filled with the appropriate filler depending on what	 
	//kind of object it is   
	//Purpose: When a user drops a brick on the grid, it should fall into the closest	
	//holders and fill theGrid object.	 
	public void Get_Closest_Holder_and_Fill_Grid(final int x, final int y, final UserObject object){	   

		int xClose = (int) Math.round( (double) (x - intGridX1 ) / intHolderWidth ) ;	
		int yClose = (int) Math.round( (double) (y - intGridY1 ) / intHolderHeight );	

		//The next 4 if will make the brick fall into place if the brick is placed   
		//just slightly off of the grid.   
		if ((xClose + object.holderSizeInX) > intGridWidth){   
			xClose = intGridWidth - object.holderSizeInX;	 
		}      
		else if (xClose < 0)   
			xClose = 0;	  

		if (yClose >= intGridHeight){	
			yClose = intGridHeight -1;	 
		}   

		else if (yClose < 0) {	 
			yClose = 0;		   
		}		  
		//Fill the holders here	in X direction
		for (int m = 0; m < object.holderSizeInX; m++){	  
			final int z = xClose + m;	  
			if ((z < intGridWidth) && (z >= 0)){   
				if (object.whichObject == BRICK)	 
					grid.theGrid[z][yClose].FullWithABrick = true;   
				else if	 (object.whichObject == TARGET_POINT)	
					grid.theGrid[z][yClose].FullWithATarget = true;  
				else if	 (object.whichObject == WEIGHT)	  
					grid.theGrid[z][yClose].FullWithAWEIGHT = true;	 
				else if (object.whichObject == GROUND)	 
					grid.theGrid[z][yClose].FullWithGround = true;   
				else if (object.whichObject == RESTRICTION)   
					grid.theGrid[z][yClose].FullWithARestriction = true;   
			}	
		}   

		//Fill the holders here in Y direction	  
		for (int m = 0; m < object.holderSizeInY; m++){	  
			final int z = yClose + m;	  
			if ((z < intGridHeight) && (z >= 0)){   
				if (object.whichObject == BRICK)	 
					grid.theGrid[xClose][z].FullWithABrick = true;   
				else if	 (object.whichObject == TARGET_POINT)	
					grid.theGrid[xClose][z].FullWithATarget = true;  
				else if	 (object.whichObject == WEIGHT)	  
					grid.theGrid[xClose][z].FullWithAWEIGHT = true;	 
				else if (object.whichObject == GROUND)	 
					grid.theGrid[xClose][z].FullWithGround = true;   
				else if (object.whichObject == RESTRICTION)   
					grid.theGrid[xClose][z].FullWithARestriction = true;   
			}	
		}   	
		final int xCoord = intGridX1 + (xClose * intHolderWidth);	   
		final int yCoord = intGridY1 + (yClose * intHolderHeight);   
		//Used to relay information to the CGI of the wherabouts of a piece, on the grid   
		object.gridX = xClose;	 
		object.gridY = yClose;	 

		object.lego1X=xCoord;
		object.lego1Y=yCoord;

		//redundant, but makes sense when you look at mouse released   
		object.beforeX = xCoord;   
		object.beforeY = yCoord;   
		//the coords of where in the grid the lego should go   
		object.nextX  = xCoord ;   
		object.nextY  = yCoord;	  


	}	

	//Returns true if A and B are true:	  
	//	A: If x is within xMin and xMax	  
	//	B: if Y is within yMin and yMaxyour x and y click  are within the xmin, xmax and    
	//	 
	//Purpose: This function helps determine if a user clicks on a certain brick   
	public boolean Contain(final int x, final int y, final int xMin, final int xMax, final int yMin, final int yMax){   

		if ((x <= xMax)	 && (x >= xMin) &&  (y <= yMax)	 && (y >= yMin))   
			return true;   
		else   
			return false;   
	}	

	public void printGrid(){ 
		for (int i =0; i< intGridWidth; i++){ 
			for (int j=0; j<intGridHeight; j++){ 
				if (grid.theGrid[i][j].FullWithABrick) 
					System.out.print("B"); 
				else if (grid.theGrid[i][j].FullWithATarget) 
					System.out.print("T"); 
				else if (grid.theGrid[i][j].FullWithGround) 
					System.out.print("G"); 
				else if (grid.theGrid[i][j].FullWithARestriction) 
					System.out.print("R"); 
				else System.out.print("0"); 

			} 
			System.out.println();	 
		} 
	} 

	//Purpose: Updates the bricks next x, and y so it can be repainted.	 xClick is subtracted because	
	//nextX must contain the coordinates of the upper left coord of the brick.	 
	//Same with nextY. xClick is just where the mouse is clicked.   
	public void updateLocation(final int x,final int y, final UserObject object) {   
		object.nextX = x - object.xclick;   
		object.nextY = y - object.yclick;   
	}	


	//Paints the top box, grid, each brick and the button.  Black brick are painted for falling bricks.	  
	//@Override
	public void paint(final Graphics g) {
		super.paint(g);
		int num_Paintable_Objects= 0;	

		g.setColor(Color.gray); 
		g.draw3DRect(0,
				0,
				canvasWidth-1,
				canvasHeight-1,
				true);

		g.draw3DRect(3,
				3,
				canvasWidth-7,
				canvasHeight-7,
				false); 

		g.draw3DRect(6,
				6,
				canvasWidth-12,
				intGridY1,
				true);

		g.draw3DRect(8,
				8,
				canvasWidth-16,
				intGridY1-4,
				false);

		//Draw The Grid
		if (showGrid){
			int vertLines=intGridX1;
			int n=1;
			g.setColor(Color.black);   
			while(vertLines<intGridX2){
				g.drawLine(vertLines, intGridY1, vertLines, intGridY2);	//vertical lines   
				vertLines = intGridX1 +(n *intHolderWidth);
				n++;
			}   

			//Horizontal
			final int x2 = vertLines - intHolderWidth;
			int m=1;
			int horizLines=0;
			while(horizLines<intGridY2){
				g.drawLine(intGridX1, horizLines + intGridY1, x2, horizLines + intGridY1);//horizontal lines	  
				horizLines = (m*intHolderHeight);
				m++;		
			}
		}
		/*To give the user the impression that all the bricks are stacked upon one another,    
	  I paint from the bottom up.  I make a new temporary array that is a complete replica of    
	  the regular array. But this one is sorted in a special way.	
	  I then paint them.	  
		 */
		UserObject Temp_Object = new UserObject();   
		final int ObjectListLength = getObjectListLength();	
		/*make a temp list of all the bricks   
	  put the bricks and the ground in the front of the temp list, including the	
	  ground if it is sposed to be painted.	 Ground is only to be painted if it has to   
	  be, indicated by grid x being -1.
		 */  
		for (int j = 0; j < ObjectListLength; j++){   
			//System.out.print(intObject_Counter);   
			//System.out.println("j "+j);   
			//System.out.println("intoBject "+intObject_Counter);   
			//System.out.println("whicObj "+Movable_Objects[j].whichObject);   
			//System.out.println("Movable_Objects[j].gridX");	
			//System.out.println("J "+j);   
			if (Movable_Objects[j].whichObject == BRICK || ((Movable_Objects[j].whichObject == GROUND) )) {	   
				Temp_Array[num_Paintable_Objects] = Movable_Objects[j];	  
				num_Paintable_Objects++;   
			}	
		}   

		/* START SORTING OF BRICKS FROM BOTTOM UP*/
		final int offSet = num_Paintable_Objects;
		//The number of bricks+ground	
		//and all the other objects on the back of the temp list   
		for (int k = 0; k < ObjectListLength; k++){   
			if (Movable_Objects[k].whichObject != BRICK	 && Movable_Objects[k].whichObject != GROUND){	  
				Temp_Array[num_Paintable_Objects] = Movable_Objects[k];	  
				num_Paintable_Objects++;   
			}	
		}   

		//sort just the bricks in order the highest ones on the grid are at the end of the list	  
		//this way the bricks get painted from the bottom.     
		//the non bricks are now on the end of the list so they are painted last too.	
		for (int l = 0; l < offSet; l++){   
			for (int j = l+1; j < offSet; j++){	   
				if (Temp_Array[l].gridY < Temp_Array[j].gridY){	  
					Temp_Object = Temp_Array[l];   
					Temp_Array[l] = Temp_Array[j];   
					Temp_Array[j] = Temp_Object;   
				}	   
			}		      
		}   

		/*End Sorting*/
		if (blnShow){//this is the boolean that gurantees that all the images load up before they are painted	
			for (int i = 0; i < num_Paintable_Objects; i++){   

				//here is where I draw all the lego pieces from the bottom up.	   
				if (Temp_Array[i].whichObject == BRICK){	   
					if (Temp_Array[i].blnIsFalling == true){   
						g.drawImage(Temp_Array[i].Break_Image, Temp_Array[i].lego1X, Temp_Array[i].lego1Y, Temp_Array[i].legoWidth, Temp_Array[i].legoHeight, this);	     
					}	
					else {
						if (Temp_Array[i].Lego_Image == null) {;
						}else	
							g.drawImage(Temp_Array[i].Lego_Image, Temp_Array[i].lego1X, Temp_Array[i].lego1Y, Temp_Array[i].legoWidth, Temp_Array[i].legoHeight, this);	    
					}	
				}else if (Temp_Array[i].whichObject == GROUND){	

					g.drawImage(Temp_Array[i].Lego_Image, Temp_Array[i].lego1X, Temp_Array[i].lego1Y, Temp_Array[i].legoWidth, Temp_Array[i].legoHeight, this);		   
					//g.drawImage(bi, 0, 0, this);	
				}else if (Temp_Array[i].whichObject == WEIGHT){
					g.setColor(Color.black);  
					final int theWidth = Temp_Array[i].legoWidth-1;
					final int theHeight = Temp_Array[i].legoHeight-1;
					//Circle
					g.setColor(Color.black);  
					g.fillOval(Temp_Array[i].lego1X+(theWidth/4), Temp_Array[i].lego1Y, theWidth/2, theWidth/2);  
					//vetical line
					g.drawLine(Temp_Array[i].lego1X+(theWidth/2), Temp_Array[i].lego1Y, Temp_Array[i].lego1X+(theWidth/2),  Temp_Array[i].lego1Y+theHeight);
					//diagnol lines
					g.drawLine(Temp_Array[i].lego1X, Temp_Array[i].lego1Y+(theHeight/2), Temp_Array[i].lego1X+(theWidth/2),Temp_Array[i].lego1Y+theHeight);
					g.drawLine(Temp_Array[i].lego1X+theWidth, Temp_Array[i].lego1Y+(theHeight/2), Temp_Array[i].lego1X+(theWidth/2),Temp_Array[i].lego1Y+theHeight); 
				}else if (Temp_Array[i].whichObject==TARGET_POINT){
					g.setColor(Color.black); 
					g.fillOval(Temp_Array[i].lego1X, Temp_Array[i].lego1Y, Temp_Array[i].legoWidth, Temp_Array[i].legoHeight);		   
				}else if (Temp_Array[i].whichObject==RESTRICTION){ 
					g.setColor(Color.black);  
					final int theWidth = Temp_Array[i].legoWidth-1;
					final int theHeight = Temp_Array[i].legoHeight-1;
					g.drawLine(Temp_Array[i].lego1X, Temp_Array[i].lego1Y, Temp_Array[i].lego1X+theWidth,Temp_Array[i].lego1Y+theHeight);
					g.drawLine(Temp_Array[i].lego1X+theWidth, Temp_Array[i].lego1Y, Temp_Array[i].lego1X, Temp_Array[i].lego1Y+theHeight);
				}	 
			}	
		}	


	}	

	int getObjectListLength(){	 
		for (int i = 0; i < intObject_Counter+5; i++){	 
			if (Movable_Objects[i] == null)   
				return i;   
		}   
		return -1;	      
	}	


	//Input nothing, returns the ith brick in the array which is being clicked on.	   
	int getClickedOnBrick (){	
		int i;	 
		for ( i = 0; i < getObjectListLength(); i++){	
			if (Movable_Objects[i].blnClickinOn)   
				return i;   
		}   
		return -1;   
	}	   


	/*Updates screen.  This program uses a buffered image so the first time a 
    new buffered image must be created.
	 */	 
	public void update( final Graphics g ) {	 
		int On_This_Brick;   
		if(firstTime){  
			//twodg = new twoDG(this);
			//bi = (BufferedImage)createImage(canvasWidth, canvasHeight);
			//big = twodg.bi.createGraphics();
			bi = createImage(canvasWidth, canvasHeight);
			big =bi.getGraphics();
			firstTime = false;
			g.setColor(Color.white);
			g.fillRect(0,0,canvasWidth,canvasHeight);
		} 
		big.setColor(Color.white);
		big.fillRect(0, 0, canvasWidth, canvasHeight);
		if ((On_This_Brick = getClickedOnBrick()) >= 0) {	
			Movable_Objects[On_This_Brick].lego1X = Movable_Objects[On_This_Brick].nextX;	
			Movable_Objects[On_This_Brick].lego1Y = Movable_Objects[On_This_Brick].nextY;	
		}
		paint(big);		  
		g.drawImage(bi, 0, 0, this);
	}

	// Returns a specific image given certain parameters about an object
	public Image get_Image(final int holderSizeInX, final int whichObject, final boolean isBreak){	
		if (whichObject == GROUND)   
			return Image_List[3].theImage;   
		else	 
			if (whichObject == FIXED)	
				return Image_List[2].theImage;	   
			else     
				if (isBreak){	
					if (holderSizeInX == 1)   
						return Image_List[10].theImage;	  
					if (holderSizeInX == 2)   
						return Image_List[11].theImage;	  
					if (holderSizeInX == 4)   
						return Image_List[12].theImage;	  
					if (holderSizeInX == 6)   
						return Image_List[13].theImage;	  
					if (holderSizeInX == 8)   
						return Image_List[14].theImage;	  
					if (holderSizeInX == 10)   
						return Image_List[15].theImage;	  
				}else {	  
					if (holderSizeInX == 1)   
						if (whichObject == RESTRICTION)	  
							return Image_List[0].theImage;   
						else	
							if (whichObject == TARGET_POINT)   
								return Image_List[1].theImage;	 
							else  
								return Image_List[4].theImage;	 
					if (holderSizeInX == 2)   
						return Image_List[5].theImage;	 
					if (holderSizeInX == 4)   
						return Image_List[6].theImage;	 
					if (holderSizeInX == 6)   
						return Image_List[7].theImage;	 
					if (holderSizeInX == 8)   
						return Image_List[8].theImage;	 
					if (holderSizeInX == 10)   
						return Image_List[9].theImage;	 
				}   
		System.err.println("Error in get_Image, holderSizeInX= " +holderSizeInX);   
		return Image_List[9].theImage;	 
	}	

	//Returns a new User_Object
	//It takes grid coordinates if it is on the grid! 
	//it takes x, y coordinates if it is a menuitem!   
	public UserObject MakeANewObject(final int whichObject, final int X, final int Y, final int holderSizeInX, final int holderSizeInY, final boolean isMenuItem){	  
		int i;
		final UserObject tmp_object = new UserObject();   
		tmp_object.whichObject = whichObject;
		tmp_object.holderSizeInX = holderSizeInX;     
		tmp_object.holderSizeInY = holderSizeInY;
		if (whichObject == BRICK){   
			tmp_object.legoWidth = (holderSizeInX * Lego_Width_Multiple);	 
			tmp_object.legoHeight = Lego_Height_Constant;   
		}else if (whichObject == GROUND){   
			tmp_object.legoWidth = (holderSizeInX * Lego_Width_Multiple);	 
			tmp_object.legoHeight = Lego_Height_Constant;   
		}else if (whichObject == TARGET_POINT){	  
			tmp_object.legoWidth = (holderSizeInX* Lego_Width_Multiple);	  
			tmp_object.legoHeight = Lego_Height_Constant;  
		}else if (whichObject == RESTRICTION){	
			tmp_object.legoWidth = (holderSizeInX* Lego_Width_Multiple);	  
			tmp_object.legoHeight = Lego_Height_Constant*holderSizeInY;     
		}else if (whichObject == WEIGHT){   
			tmp_object.legoWidth = Lego_Width_Multiple;	
			tmp_object.weight = holderSizeInX;
			tmp_object.holderSizeInX = 1;
			for (i=0; (i < applet.weights.length); i++){
				if (holderSizeInY == (int) Integer.parseInt(applet.weights[i]))
					tmp_object.legoHeight = (i+1) *	 Lego_Height_Constant;
			}
		}	 

		if (isMenuItem){   
			tmp_object.gridX = -1;   
			tmp_object.gridY = -1;   
			tmp_object.initX = X;   
			tmp_object.initY = Y;     
			tmp_object.lego1X = tmp_object.initX;   
			tmp_object.lego1Y = tmp_object.initY;    
		}   
		else   
		{	 
			tmp_object.gridX = X;	
			tmp_object.gridY = Y;	
			tmp_object.initX = intGridX1 + (X * intHolderWidth);   
			tmp_object.initY = intGridY1 + (Y * intHolderHeight);	  
			tmp_object.lego1X = tmp_object.initX;	
			tmp_object.lego1Y = tmp_object.initY;	
		}	


		tmp_object.Lego_Image = get_Image(tmp_object.holderSizeInX, whichObject, false);   
		tmp_object.Break_Image = get_Image(tmp_object.holderSizeInX, whichObject, true);   

		tmp_object.xclick = 0;	 
		tmp_object.yclick = 0;

		//Booleans   
		tmp_object.blnClickinOn = false;   
		tmp_object.blnNewUser_Object = isMenuItem;   
		tmp_object.blnIsFalling = false;   

		return tmp_object;   

	}	 


	public void EmptyGrid_and_List(){	  
		for (int j = menu_Items; j<getObjectListLength(); j++){		
			if (Movable_Objects[j].gridX >= 0){	  
				emptyGridHolders( Movable_Objects[j]);	 
				Movable_Objects [j]= null;  
			}	   
		}	   
		intObject_Counter = menu_Items;		      
		blnMaking_New_Brick = false;
	}	


	public void printList(){ 
		System.out.println();	
		for (int i = 0; i<intObject_Counter; i++){   
			System.out.print("ith "+i); 
			System.out.print(" object "+Movable_Objects[i].whichObject);   
			System.out.print(" grix "+Movable_Objects[i].gridX);   
			System.out.println();						 
		}    
		System.out.println("LISTDONE"); 
	}	


	public void Empty_Bricks(final boolean REPAINT){
		final UserObject tmp_list[] = new UserObject[maxObjects];	  
		int counter =0;
		for (int i=menu_Items; i<intObject_Counter ; i++){ 
			if (Movable_Objects[i].whichObject == WEIGHT || Movable_Objects[i].whichObject == RESTRICTION
					|| Movable_Objects[i].whichObject == TARGET_POINT){
				tmp_list[counter]= Movable_Objects[i];
				counter++;
			}
		}
		myInit();
		appendList(tmp_list, counter, Movable_Objects, menu_Items); 

		if (REPAINT)
			repaint();
	}

	public void Update_Structure(int objectCount, final ioObject[] list, final boolean ONLY_UPDATE_BRICKS) {	  
		final UserObject tmp_list[] = new UserObject[maxObjects];   
		int i;
		int tempCounter=0;
		//System.out.println("CGI SENT->");
		if (ONLY_UPDATE_BRICKS){
			Empty_Bricks(false) ;
			//System.out.println("objCount="+objectCount);
			for (i=0; i < objectCount; i++){  
				if (list[i].kind == GROUND || list[i].kind == BRICK){
					tmp_list[tempCounter] = MakeANewObject(list[i].kind,	  
							list[i].x,   
							(intGridHeight - 1) - list[i].y,	   
							list[i].size,
							1,	 
							false);	 
					/*
		    System.out.print("whicObj " + tmp_list[tempCounter].whichObject+ " "); 
		    System.out.print("girdX " + tmp_list[tempCounter].gridX+ " "); 
		    System.out.print("gridy " +(intGridHeight - tmp_list[tempCounter].gridY - 1)+ " ");  
		    if (tmp_list[tempCounter].whichObject==WEIGHT){
		      System.out.print("weight " + tmp_list[tempCounter].weight); 
		    }else System.out.print("holderSizeInX " + tmp_list[tempCounter].holderSizeInX); 
		    System.out.println(" "); 
					 */
					tempCounter++;
				}
			}
			objectCount = tempCounter;
		}else{
			myInit();
			for (i=0; i < objectCount; i++){  
				tmp_list[i] = MakeANewObject(list[i].kind,	  
						list[i].x,   
						(intGridHeight - 1) - list[i].y,	   
						list[i].size,
						1,	 
						false);	 
				//System.out.print("whicObj " + tmp_list[i].whichObject+ " "); 
				//System.out.print("girdX " + tmp_list[i].gridX+ " "); 
				//System.out.print("gridy " +(intGridHeight - tmp_list[i].gridY - 1)+ " ");  
				//if (tmp_list[i].whichObject==WEIGHT){
				//    System.out.print("weight " + tmp_list[i].weight); 
				//}else System.out.print("holderSizeInX " + tmp_list[i].holderSizeInX); 
				//System.out.println(" "); 
			}				 
		}


		//Fill the grid with the new Stuff from the server
		for (i=0; i <objectCount; i++) {  
			Get_Closest_Holder_and_Fill_Grid(tmp_list[i].lego1X, tmp_list[i].lego1Y, tmp_list[i]);
		}   


		applet.statusLine("Structure loaded. Continue Building.");
		//Add the bricks and stuff from server to the Brick List
		appendList(tmp_list, objectCount, Movable_Objects, intObject_Counter);	 

	}	 



	public void appendList(final UserObject tmpList[], final int tmpListLength, final UserObject realList[], int counter){	  
		int i = 0;	   
		while (i< tmpListLength){   
			realList[counter] = tmpList[i];   
			counter++;	   
			i++;	 
		}    
		//System.out.println("cnter convert1 "+intObject_Counter);   
		intObject_Counter = counter;   

		//System.out.println("cnter convert2 "+intObject_Counter);   
	}	



	public int getNumOfBricksOnGrid(){ 
		int counter = 0; 
		for (int i=0; i< intObject_Counter; i++) { 
			if (Movable_Objects[i].whichObject == BRICK	 && (Movable_Objects[i].gridX >= 0)) 
				counter++;		 
		} 
		return counter; 

	} 

	int getLengthWithRestrictions(){
		int moreRestrict=0;
		for (int i=0; i< intObject_Counter; i++) { 
			if (Movable_Objects[i].whichObject== RESTRICTION  && (Movable_Objects[i].blnNewUser_Object==false)){
				for (int j=Movable_Objects[i].holderSizeInX; j>1; j--){
					moreRestrict++;
				}	 
				for (int j=Movable_Objects[i].holderSizeInY; j>1; j--){
					moreRestrict++;
				}
			}
		}
		return ((intObject_Counter-menu_Items)+moreRestrict);
	}

	int[] printStructure(final PrintStream outStream) {
		// send no of bricks 
		//send "ground" specs 

		int i; 
		final int number_list[] = new int[intObject_Counter]; 
		int count = 0; 

		// Number list is a list of the index number of the bricks that are on the grid, 
		// and not on the top part.  
		// Start on number of non bricks, because the those are in the brick list but are not bricks. 


		//These print statements mimic exactly what the cgi looks like it is getting
		applet.statusLine("Sending structure...");
		//System.out.println("Sending to CGI->");
		//System.out.print(GRID_SIZE+" "+xMin+" "+(xMin+intGridWidth)+" "+yMin+" "+(yMin+intGridHeight)+" ");
		//System.out.println(" ");
		//System.out.println("Stuff on Grid "+ (intObject_Counter - menu_Items)); 

		outStream.print(GRID_SIZE+" "+xMin+" "+(xMin+intGridWidth)+" "+yMin+" "+(yMin+intGridHeight)+" ");
		outStream.println(" ");
		outStream.println(getLengthWithRestrictions());
		//(intObject_Counter - menu_Items); 

		//User_Object Temp_Object = new User_Object();	 
		//int ObjectListLength = getObjectListLength();

		for (i=0; i< intObject_Counter; i++) { 
			if (Movable_Objects[i].gridX >= 0){//(-1 if not on grid) 
				number_list[count]=i; 
				count++; 
				Movable_Objects[i].blnIsFalling=false; 
				Movable_Objects[i].print(outStream);
			} 
		}
		applet.statusLine("structure sent.");
		return number_list;
	}

	/**************************** Communication with CGI's below*****************/

	//Called when Evolve is pressed
	public void evolve() {
		try 
		{ //put this back in 
			final TalkToBackend connection = new TalkToBackend("evol"+LegoApplet.extension);
			final PrintStream outStream = new PrintStream(connection.getOutputStream());
			printStructure(outStream);
			outStream.close();  
			applet.statusLine("Evolving, please wait");
			final BufferedReader in = new BufferedReader(
					new InputStreamReader(
							connection.getInputStream()));
			final StructParser p = new StructParser(in);
			if (p.valid) 
				Update_Structure(p.objectCount,p.objectList,true); 
			else 
				System.out.println("Error: "+p.errorMsg);
			in.close(); 
			loading = false; 
			repaint();
		} 
		catch ( final MalformedURLException me ) 
		{ 
			System.err.println( "MalformedURLException: " + me ); 
		} 
		catch ( final IOException ioe ) 
		{ 
			System.err.println( "IOException: " + ioe ); 
		} 

	}

	//Sends the the bricks and grid to the server.  The server tells the applet if the structure 
	//stays up or falls. 
	public void lTest() {	 
		try 
		{ //put this back in 
			final TalkToBackend connection = new TalkToBackend("mtest"+LegoApplet.extension);
			final PrintStream outStream= new PrintStream(connection.getOutputStream());
			final int[] number_list = printStructure(outStream);
			outStream.close(); 
			final BufferedReader in = new BufferedReader(
					new InputStreamReader(connection.getInputStream())); 

			final String inputLine = in.readLine();//need to make it read everything, not just the line 
			//might cause problems if input is really large 
			//System.out.println("From CGI="+inputLine);
			final StringTokenizer st = new StringTokenizer(inputLine); 
			final int numberOfTokens = st.countTokens(); 
			final int[] input_list = new int[numberOfTokens];
			final String[] tempInput_list = new String[numberOfTokens];
			String nextT; 
			int nextInt;
			int i;
			if (numberOfTokens == 0) {
				System.out.println("CGI RETURNED NULL");
				applet.statusLine("Error: No reply from simulator");
			}
			else{
				for (i=0;i < numberOfTokens; i++){	 
					nextT = st.nextToken(); 
					tempInput_list[i]  = nextT; 
				} 
				if ((int) Integer.parseInt(tempInput_list[0]) == -1)
					System.err.println("error, CGI returned -1");
				else{
					for (i=0;i < numberOfTokens; i++){	 
						nextInt = (int) Integer.parseInt(tempInput_list[i]); 
						input_list[i]	= nextInt; 
					} 

					//(int) Integer.parseInt(
					//input line can be -1, for erorr, 0 for good, and a series of numbers that 
					//indicate which bricks have fallen 
					if (null == inputLine) { 
						System.err.println("error: no output from ltest.cgi"); 
					} 
					else { 
						if (numberOfTokens == 1){ 
							if (input_list[0] == 0){    
								applet.statusLine("structure is stable");
							} 
							else { 
								System.out.println("error, "+inputLine+" returned");  //error 
								applet.statusLine("error in simulator: "+inputLine);
							} 
						} else { 
							applet.statusLine("structure is unstable");
							//flag bad legos 
							for (i=1;i < numberOfTokens; i++){ 


								if (Movable_Objects[number_list[input_list[i]]].whichObject == BRICK) 
									Movable_Objects[number_list[input_list[i]]].blnIsFalling = true; 
							}  
						}
						//System.out.println("cgi returned->"+inputLine); 
					} 

					/*Close the connection and then make sure the whole screen is repainted.  
			  Even More efficiant would be a repaint that only paints the grid.*/ 
					in.close();	   
				}
			}
			repaint(); 
		}  
		catch ( final MalformedURLException me ) 
		{ 
			System.err.println( "MalformedURLException: " + me ); 
		} 
		catch ( final IOException ioe ) 
		{ 
			System.err.println( "IOException: " + ioe ); 
		} 
	} 

	//Called when "Save" button is pressed
	public void save_all() { 
		repaint();	
		try 
		{ //put this back in 
			final TalkToBackend connection = new TalkToBackend("save"+LegoApplet.extension);
			final PrintStream outStream = new PrintStream(connection.getOutputStream());
			printStructure(outStream);
			outStream.close(); 
			final BufferedReader in = new BufferedReader(
					new InputStreamReader(connection.getInputStream()));
			in.close(); 
		} 
		catch ( final MalformedURLException me ) 
		{ 
			System.err.println( "MalformedURLException: " + me ); 
		} 
		catch (final IOException ioe) 
		{ 
			System.err.println( "IOException: " + ioe ); 
		} 
	}

	//When  loading a saved structure, called when "Load" button is pressed
	public void load_saved() { 
		repaint();	
		try 
		{ //put this back in 

			final TalkToBackend connection = new TalkToBackend("load"+LegoApplet.extension);

			final PrintStream outStream = new PrintStream(connection.getOutputStream());
			outStream.close(); 
			final BufferedReader in = new BufferedReader(
					new InputStreamReader(connection.getInputStream()));

			final StructParser p = new StructParser(in);
			if (p.valid)     	
				Update_Structure(p.objectCount,p.objectList,false); 
			else
				System.out.println("Error: "+p.errorMsg);

			in.close(); 

			loading = false; 
			repaint();  
		} 
		catch ( final MalformedURLException me ) 
		{ 
			System.err.println( "MalformedURLException: " + me ); 
		} 
		catch ( final IOException ioe ) 
		{ 
			System.err.println( "IOException: " + ioe ); 
		} 

	}

	//Not implemented
	public void Help(){
		;	
	}

	public void clearAllBricks(){
		myInit();
		repaint();
	}


	public void changeRestrictionSize(final int size){
		if (size == 1){
			for (int i=0; i<intObject_Counter; i++){
				if ((Movable_Objects[i].whichObject == RESTRICTION) && Movable_Objects[i].blnNewUser_Object){	
					Movable_Objects[i].holderSizeInX =1;
					Movable_Objects[i].holderSizeInY=1;
					Movable_Objects[i].legoWidth = (Movable_Objects[i].holderSizeInX* Lego_Width_Multiple);	  
					Movable_Objects[i].legoHeight = Lego_Height_Constant*Movable_Objects[i].holderSizeInY;   
					repaint();
					break; 
				}
			}
		}else if (size == 2){
			for (int i=0; i<intObject_Counter; i++){
				if ((Movable_Objects[i].whichObject == RESTRICTION) && Movable_Objects[i].blnNewUser_Object){	
					Movable_Objects[i].holderSizeInX =2;
					Movable_Objects[i].holderSizeInY=1;
					Movable_Objects[i].legoWidth = (Movable_Objects[i].holderSizeInX* Lego_Width_Multiple);	  
					Movable_Objects[i].legoHeight = Lego_Height_Constant*Movable_Objects[i].holderSizeInY; 
					repaint();
					break;
				}
			}
		}
		else if (size == 3){
			for (int i=0; i<intObject_Counter; i++){
				if ((Movable_Objects[i].whichObject == RESTRICTION) && Movable_Objects[i].blnNewUser_Object){	
					Movable_Objects[i].holderSizeInX =1;
					Movable_Objects[i].holderSizeInY=2;
					Movable_Objects[i].legoWidth = (Movable_Objects[i].holderSizeInX* Lego_Width_Multiple);	  
					Movable_Objects[i].legoHeight = Lego_Height_Constant*Movable_Objects[i].holderSizeInY; 
					repaint();
					break;
				}
			}
		}		
	}

	//Changing the Grid Size
	public void changeGridSize(final int size){
		//System.out.println("objCout="+intObject_Counter+" grIn "+groundIndex);
		if (intObject_Counter != groundIndex+1){
			applet.statusLine("Only Resize In Begining");
		}else if (size == 1){

			intGridWidth = 40;// (int) Math.floor(intGridWidth*1.5);
			intGridHeight = 20;//(int) Math.floor(intGridHeight*1.5); 
			//grid = new Grid(lc.canvas);
			intHolderWidth = 18;//(int) Math.floor(intHolderWidth/1.5);
			intHolderHeight =18;//(int) Math.floor(intHolderHeight/1.5);  
			Lego_Width_Multiple = intHolderWidth;
			Lego_Height_Constant = intHolderHeight+2;
			for (int i=0; i<getObjectListLength();i++){  
				//if (Movable_Objects[i].whichObject == BRICK	 || Movable_Objects[i].whichObject == GROUND || Movable_Objects[i].whichObject == TARGET_POINT || Movable_Objects[i].whichObject == RESTRICTION){    
				Movable_Objects[i].legoHeight = Lego_Height_Constant;
				Movable_Objects[i].legoWidth = Movable_Objects[i].holderSizeInX* intHolderWidth;
				//}
			} 
			repaint();
		}else if (size == 2){
			intGridWidth = 60;// (int) Math.floor(intGridWidth*1.5);
			intGridHeight = 30;//(int) Math.floor(intGridHeight*1.5); 
			grid = new Grid(applet.canvas);

			intHolderWidth = 12;//(int) Math.floor(intHolderWidth/1.5);
			intHolderHeight =12;//(int) Math.floor(intHolderHeight/1.5);  
			Lego_Width_Multiple = intHolderWidth;
			Lego_Height_Constant = intHolderHeight+2;
			for (int i=0; i<getObjectListLength();i++){

				//if (Movable_Objects[i].whichObject == BRICK	 || Movable_Objects[i].whichObject == GROUND || Movable_Objects[i].whichObject == TARGET_POINT || Movable_Objects[i].whichObject == RESTRICTION){    
				Movable_Objects[i].legoHeight = Lego_Height_Constant;
				Movable_Objects[i].legoWidth = Movable_Objects[i].holderSizeInX* intHolderWidth;
				//}
			} 
			//intGridX2 =	intGridWidth * intHolderWidth-8;
			repaint();
		}else if (size == 3){
			intGridWidth = 80;// (int) Math.floor(intGridWidth*2);
			intGridHeight = 40;//(int) Math.floor(intGridHeight*2); 
			grid = new Grid(applet.canvas);

			intHolderWidth = 9;//(int) Math.floor(intHolderWidth/2);
			intHolderHeight =9;//(int) Math.floor(intHolderHeight/2);  
			Lego_Width_Multiple = intHolderWidth;
			Lego_Height_Constant = intHolderHeight+2;
			for (int i=0; i<getObjectListLength();i++){

				//if (Movable_Objects[i].whichObject == BRICK	 || Movable_Objects[i].whichObject == GROUND || Movable_Objects[i].whichObject == TARGET_POINT || Movable_Objects[i].whichObject == RESTRICTION){    
				Movable_Objects[i].legoHeight = Lego_Height_Constant;
				Movable_Objects[i].legoWidth = Movable_Objects[i].holderSizeInX* intHolderWidth;
				//}
			} 
			//intGridX2 =	intGridWidth * intHolderWidth-8;
			repaint();

		}

	}		


	public boolean mouseMove(final Event e,final int x,final int y){return true;}	  
	public boolean mouseEnter(final Event e,final int x,final int y){return true;}   
	public boolean mouseExit(final Event e,final int x,final int y){return true;}	  


	/*Class used in assisting with connections to CGI's*/	
	class TalkToBackend {
		public boolean open=false;
		public URL url;
		public URLConnection cgi=null;
		public Process exec=null;

		public TalkToBackend(final String name) throws IOException {
			//System.out.println("Connecting 2 "+name);
			try {
				if (applet.isRemoteMode()) {
					url=new URL(LegoApplet.BaseURL,name);
					cgi=url.openConnection();
					cgi.setDoOutput(true);
				}
				else {
					exec = Runtime.getRuntime().exec(applet.getExecPrefix()+name);
				}
				open=true;
			}
			catch (final IOException e) {
				applet.statusLine("Failed to connect to server! "+e);
				throw e;
			}
		}

		public OutputStream getOutputStream() throws IOException {
			if (open) {
				if (applet.isRemoteMode()) return cgi.getOutputStream();
				else return exec.getOutputStream();
			}
			else return null;
		}
		public InputStream getInputStream () throws IOException {
			if (open) {
				if (applet.isRemoteMode()) return cgi.getInputStream();
				else return exec.getInputStream();
			}
			else return null;
		}

	}

}

class ImageHolder { 
	Image theImage = null; 
} 

class UserObject {//make this one movable objects   
	//ground =0, brick=1, TARGET_POINT = 2, restriction = 3	
	int whichObject;   

	boolean blnClickinOn = false;   
	boolean blnNewUser_Object = true;	
	boolean blnIsFalling = false;   
	Image Lego_Image;	
	Image Break_Image;	 


	int xclick = 0 ;   
	int yclick = 0 ;   
	//used with the clipboard	
	int lego1X;	  
	int lego1Y;	  
	//If grix and gridy are -1, then this piece is off of the grid.   
	int gridX  = -1;   
	int gridY = -1;   
	int nextX;	 
	int nextY;	 

	int legoHeight;   
	int legoWidth;   
	int beforeX;   
	int beforeY;   
	int initX;	 
	int initY;	 

	//this is the number of holders it takes up.   
	//It is also known as the numbe of nodes it has on top,   
	int holderSizeInX;
	int holderSizeInY = 1;   

	/*The Weight of the Arrows
      If it is -1, this means the weight hasn't been set yet.
      However, every arrow on the grid must have some weight to it*/
	int weight = -1;

	//Sending info to CGI
	public void print(PrintStream out) {
		if (whichObject == LegoCanvas.RESTRICTION)
			printRestriction(out);
		else if (whichObject == LegoCanvas.WEIGHT){
			out.print(whichObject+" "+gridX+" "+(LegoCanvas.intGridHeight - gridY - 1) 
					+" "+weight);
			out.println(" ");
			//System.out.print("whicObj " +whichObject+ " "); 
			//System.out.print("girdX " + gridX+ " "); 
			//System.out.print("gridy " +(legoForConf.intGridHeight  - gridY - 1)+ " ");  
			//System.out.print("weight " + weight); 
			//System.out.println(" "); 
		}else{
			out.print(whichObject+" "+gridX+" "+(LegoCanvas.intGridHeight - gridY - 1) 
					+" "+holderSizeInX);
			out.println(" ");
			//System.out.print("whicObj " +whichObject+ " "); 
			//System.out.print("girdX " + gridX+ " "); 
			//System.out.print("gridy " +(legoForConf.intGridHeight  - gridY - 1)+ " ");  
			//System.out.print("holderSizeInX " + holderSizeInX); 
			//System.out.println(" "); 
		}
	}


	//For sending info on location of restriction points to CGI 
	public void printRestriction(PrintStream out){
		//System.out.println("Restriction= ");
		out.print(whichObject+" "+gridX+" "+(LegoCanvas.intGridHeight - gridY - 1) 
				+" "+1);
		out.println(" ");
		//System.out.println(whichObject+" "+gridX+" "+(legoForConf.intGridHeight - gridY - 1) 
		//		   +" "+1+" ");
		if ((holderSizeInX>1)||(holderSizeInY>1)){
			for (int i=1;i<holderSizeInX;i++){
				out.print(whichObject+" "+(gridX+i)+" "+(LegoCanvas.intGridHeight - gridY - 1) 
						+" "+1);
				out.println(" ");
				//System.out.println(whichObject+" "+(gridX+i)+" "+(legoForConf.intGridHeight - gridY - 1)
				//+" "+1+" ");
			}
			for (int i=1;i<holderSizeInY;i++){
				out.print(whichObject+" "+gridX+" "+((LegoCanvas.intGridHeight - gridY - 1)-i) 
						+" "+1);
				out.println(" ");
				//System.out.println(whichObject+" "+gridX+" "+((legoForConf.intGridHeight - gridY - 1)-i) 
				//+" "+1+" ");
			}
		}
	}
}   

//The Grid
class Grid{   
	//if a piece is dragged close enough to a box, and then the mouse is unclicked, then it should stay there!	 
	public Holder theGrid[][];	

	public Grid(LegoCanvas lfc){   
		theGrid= new Holder[LegoCanvas.intGridWidth][LegoCanvas.intGridWidth - 1];  
		for (int j=0; j < LegoCanvas.intGridWidth; j++){	
			for (int k = 0; k< (LegoCanvas.intGridHeight); k++){	
				theGrid[j][k] = new Holder();	
			}	
		}   
	}	

}   

//The Trash image,    
class Trash {	
	Image trashImage;
}   


//Each grid space is a Holder.  It can be filled with various things
class Holder   
{   

	boolean FullWithABrick = false;   
	boolean FullWithATarget = false;   
	boolean FullWithGround = false;   
	boolean FullWithARestriction = false;
	boolean FullWithAWEIGHT = false;   
}   

